@@ -1,5 +1,6 @@
 package Benno::Mailfile;
 use Digest::SHA qw(sha256);
+use Benno;
 use strict;
 
 =head1 NAME
@@ -11,7 +12,7 @@ Benno::Mailfile - Class representing EML file from repo
 
     use Benno::Mailfile;
 
-    my $MailFile = new Benno::Mailfile($boxid,$filepath, @secretheader);
+    my $MailFile = new Benno::Mailfile($boxid,$filepath,@secretheader);
     open (my $fh, ">/path/to/emlfile.eml");
     print $fh $MailFile->eml
     close $fh;
@@ -24,7 +25,7 @@ Benno::Mailfile - Class representing EML file from repo
 
 =head2 new($bennofile)
 
-    my $MailFile = new Benno::Mailfile($boxid,$bennofile);
+    my $MailFile = new Benno::Mailfile($boxid,$bennofile,$secretheader);
 
 =cut
 sub new
@@ -45,21 +46,20 @@ sub new
         recipients   => [],
         scid         => [],
         secretheader => \@secretheader,
-        scid_secret  => 1,       # SCID not secret header
         checksum     => '',
         mailcontent  => '',
     };
     bless $self, $class;
 
     $self->_init($bennofile);
-    $self->{checksum} or die "NOT VALID\n";
+    $self->{checksum} or die "MAILFILE NOT VALID\n";
 
     return $self;
 }
 
 =head2 eml()
 
-    Return emlfile string.
+    Return emlfile string with secretheaders.
 
 =cut
 sub eml
@@ -67,19 +67,19 @@ sub eml
     my ($self) = @_;
     my $eml;
 
-    unless (grep /^X-REAL-MAILFROM$/, @{$self->{secretheader}}) {
+    if (grep /^X-REAL-MAILFROM$/, @{$self->{secretheader}}) {
         foreach my $sender (@{$self->{sender}}) {
-            $eml .= "X-REAL-MAILFROM: $sender\r\n";
+            $eml .= "X-REAL-MAILFROM: $sender\n";
         }
     }
-    unless (grep /^X-REAL-RCPTTO$/, @{$self->{secretheader}}) {
+    if (grep /^X-REAL-RCPTTO$/, @{$self->{secretheader}}) {
         foreach my $recipient (@{$self->{recipients}}) {
-            $eml .= "X-REAL-RCPTTO: $recipient\r\n";
+            $eml .= "X-REAL-RCPTTO: $recipient\n";
         }
     }
-    unless (grep /^X-BENNO-SCID$/, @{$self->{secretheader}}) {
+    if (grep /^X-BENNO-SCID$/, @{$self->{secretheader}}) {
         foreach my $scid (@{$self->{scid}}) {
-            $eml .= "X-BENNO-SCID: $scid\r\n";
+            $eml .= "X-BENNO-SCID: $scid\n";
         }
     }
 
@@ -139,7 +139,7 @@ sub export
 
     print $tmpfh $self->eml;
 
-    close $tmpfh or die print STDERR "Cannot close $tmpfile. $!\n";
+    close $tmpfh or die "Cannot close $tmpfile. $!\n";
 
     if (link($tmpfile, $emlfile)) {
         unlink $tmpfile;
@@ -181,7 +181,7 @@ sub _init
     }
 
     my $error;
-    my $headerdigest;
+    my $metadatadigest = '';
     foreach my $line (<$fh>) {
         if ($line =~ $defective_re) {
             $self->defective = 1;
@@ -199,7 +199,7 @@ sub _init
 
 
         if ($line =~ $hash_re) {
-            $headerdigest = $1;
+            $metadatadigest = $1;
 
             foreach my $address (@sender_headers) {
                 $address =~ s/[<>]//g;
@@ -217,20 +217,20 @@ sub _init
 
             next;
         }
-        $headerdigest || next;
+        $metadatadigest || next;
 
-        if ($line =~ $mail_scid) {  $self->{scid_secret} = 0; }
         $self->{mailcontent} .= $line;
     }
 
-    my $ctx = Digest::SHA->new('256');
-    my $digest = $ctx->add($self->{mailcontent})->hexdigest;
-    my $filedigest = uc $digest;
-
-    if ($filedigest ne $digest) {
-        die "CHECKSUM MISMATCH\n";
+    my $calcdigest = Benno->digest($self->{mailcontent},@{$self->{secretheader}});
+    if ($metadatadigest eq $calcdigest) {
+        $self->{checksum} = $calcdigest;
+        return;
     }
 
+    print "DEBUG metadigest: $metadatadigest\n" if $ENV{DEBUG};
+    print "DEBUG calcdigest: $calcdigest\n"     if $ENV{DEBUG};
+    die "CHECKSUM MISMATCH\n";
 }
 
 
